program ScanDFM;
// Public Domain version 1.03 (c) 2011 by Bob Swart (aka Dr.Bob - www.drbob42.com)
// Needs Delphi 2007 or higher to compile
{$APPTYPE CONSOLE}
{$I-}
uses
  Classes,
  SysUtils;

var
  Count: Integer = 0; // number of DFM files found (should be Scanned + Skipped)
  Scanned: Integer = 0; // number of DFM files scanned
  Skipped: Integer = 0; // number of DFM files skipped


function AnalyzeDirectory(const Path: String): Integer; // returns the number of issues found
const
  SuspiciousModalResult = 8; // mrAll in pre-XE2, mrClose in XE2
const
  strModalResult = 'ModalResult =';
  strObjectName  = 'object ';
var
  SRec: TSearchRec;
  Str: TStringList;
  ComponentName,S: String;
  FileFoundInDirectory: Boolean;
  ModalResultFoundInFile: Boolean;
  ModalResult,i: Integer;

  function CheckBinary(const FileName: String): Boolean; 
  var
    F: File;
    Buffer: array[0..2] of Byte; // $FF $0A $00
  begin
    Assign(f,FileName);
    FileMode := fmOpenRead;
    try 
      Reset(f,1);
      if IOResult = 0 then 
        BlockRead(f,Buffer,SizeOf(Buffer))
    finally
      Close(f)
    end;
    Result := (IOResult <> 0) or ((Buffer[0] = $FF) and (Buffer[1] = $0A) and (Buffer[2] = $00))
  end;

begin
  Result := 0;
  Str := TStringList.Create;
  FileFoundInDirectory := False;
  try
    if FindFirst('*.dfm',faAnyFile, SRec) = 0 then
    repeat
      if (SRec.Attr <> faDirectory) then
      begin
        Inc(Count);
        if CheckBinary(SRec.Name) then
        begin
          Inc(Skipped);
          writeln(Path, '\', SRec.Name, ': binary DFM file (skipped).')
        end
        else
        begin
          Inc(Scanned);
          Str.LoadFromFile(SRec.Name);
          ModalResultFoundInFile := False;
          for S in Str do // does not compile in Delphi 7, but does in Delphi 2007
          begin
            i := Pos(strObjectName, S);
            if (i > 0) then // new object found, get name
            begin
              ComponentName := Copy(S,i+Length(strObjectName));
              Delete(ComponentName,Pos(':',ComponentName),Length(ComponentName))
            end
            else // not a new object, look at property name
            begin
              i := Pos(strModalResult, S);
              if (i > 0) then // ModalResult found, get value
              begin
                ModalResult := StrToIntDef(Copy(S,i+Length(strModalResult)+1,3),0);
                if ModalResult >= SuspiciousModalResult then // ModalResult of 8 or higher are different in XE2 vs. prior
                begin
                  Result := Result + 1; // another suspicious ModalResult found
                  if (not FileFoundInDirectory) and (Length(Path) > 1) then // first file in directory
                  begin
                    writeln;
                    writeln(Path,':');
                    for i:=1 to Length(Path) do write('=');
                    writeln('=')
                  end;
                  FileFoundInDirectory := True;
                  if not ModalResultFoundInFile then // first ModalResult in file
                  begin
                    writeln(SRec.Name);
                    writeln('  // add to FormCreate of ', ChangeFileExt(SRec.Name,'.pas'))
                  end;
                  ModalResultFoundInFile := True;
                  write('  '); // indenting in logfile
                  case ModalResult of
                     1: writeln(ComponentName,'.ModalResult := mrOK; // ', Trim(S));
                     2: writeln(ComponentName,'.ModalResult := mrCancel; // ', Trim(S));
                     3: writeln(ComponentName,'.ModalResult := mrAbort; // ', Trim(S));
                     4: writeln(ComponentName,'.ModalResult := mrRetry; // ', Trim(S));
                     5: writeln(ComponentName,'.ModalResult := mrIgnore; // ', Trim(S));
                     6: writeln(ComponentName,'.ModalResult := mrYes; // ', Trim(S));
                     7: writeln(ComponentName,'.ModalResult := mrNo; // ', Trim(S));
                     8: writeln(ComponentName,'.ModalResult := mrAll; // ', Trim(S));
                     9: writeln(ComponentName,'.ModalResult := mrNoToAll; // ', Trim(S));
                    10: writeln(ComponentName,'.ModalResult := mrYesToAll; // ', Trim(S));
                    11: writeln(ComponentName,'.ModalResult := mrClose; // ', Trim(S));
                    else
                      writeln('Note: this DFM file is probably already in XE2 format! ',
                        ComponentName,'.', Trim(S))
                  end
                end
              end
            end
          end
        end
      end
    until FindNext(SRec) <> 0;

    if FindFirst('*.*',faDirectory, SRec) = 0 then
    repeat
      if ((SRec.Attr and faDirectory) = faDirectory) and (SRec.Name[1] <> '.') then
      try
        ChDir(SRec.Name);
        Result := Result + AnalyzeDirectory(Path + '\' + SRec.Name)
      finally
        ChDir('..')
      end
    until FindNext(SRec) <> 0;

    FindClose(SRec)
  finally
    Str.Free
  end
end;

var
  Suspicious: Integer;
begin
  writeln('ScanDFM v1.02 - scanning DFM files for suspicious ModalResult values');
  writeln;
  try
    Suspicious := AnalyzeDirectory('.');
    writeln;
    writeln(Count,' DFM files found, ', Scanned,' scanned, ', Skipped,' skipped (binary or access denied).');
    writeln(Suspicious,' suspicious ModalResult values found.')
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message)
  end
end.
